package se.paddbacken.relayservice;

import java.util.Arrays;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.RequestLogHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.codahale.metrics.health.HealthCheck;
import com.pi4j.io.gpio.GpioController;
import com.pi4j.io.gpio.GpioFactory;
import com.pi4j.io.gpio.GpioPinDigitalOutput;
import com.pi4j.io.gpio.Pin;
import com.pi4j.io.gpio.PinState;
import com.pi4j.io.gpio.RaspiPin;

import io.dropwizard.Application;
import io.dropwizard.Configuration;
import io.dropwizard.jetty.HttpConnectorFactory;
import io.dropwizard.lifecycle.ServerLifecycleListener;
import io.dropwizard.server.DefaultServerFactory;
import io.dropwizard.setup.Environment;
import se.paddbacken.relayservice.scheduling.LightsSchedulerResource;
import se.paddbacken.relayservice.ui.RelayServiceUIResource;
import se.paddbacken.relayservice.ui.SysInfoResource;

public class RelayServiceApplication extends Application<Configuration> {
	private static final Logger LOGGER = LoggerFactory.getLogger(RelayServiceApplication.class);
	protected static final String APPLICATION_NAME = "RelayService";
	protected static final String HEALTH_CHECK_NAME = "RelayServiceHealthCheck";
	
	public static void main(String[] args) throws Exception {
        new RelayServiceApplication().run(args.length == 0
        		? new String[]{"server"} : args);
    }

	
    @Override
    public String getName() {
        return APPLICATION_NAME;
    }

    @Override
    public void run(Configuration configuration, Environment environment) {
    	// Socket permission denied? Change port to >1000
    	((HttpConnectorFactory) ((DefaultServerFactory) configuration.getServerFactory())
    			.getApplicationConnectors().get(0)).setPort(80);
    	 
    	environment.lifecycle().addServerLifecycleListener(new ServerLifecycleListener() {
			@Override
			public void serverStarted(Server server) {
				try {
					LOGGER.info("Switching off request logging");
					server.getChildHandlerByClass(RequestLogHandler.class).getRequestLog().stop();
					LOGGER.info("Switched off request logging");
				} catch (Exception e) {
					throw new RuntimeException("Unable to stop request log", e);
				}
			}
		});
    	environment.healthChecks().register(HEALTH_CHECK_NAME, new HealthCheck(){
			@Override
			protected Result check() throws Exception {
				return Result.healthy();
			}
    		
    	});

    	final GpioController gpioController = createGpioController();
    	Runtime.getRuntime().addShutdownHook(new Thread() {
    		@Override
    		public void run() {
    			gpioController.shutdown();
    		}
    	});
    	RelaysResource relaysResource = new RelaysResource(Arrays.asList(
    			createGpioPinDigitalOutput(gpioController, RaspiPin.GPIO_00),
    			createGpioPinDigitalOutput(gpioController, RaspiPin.GPIO_02)));
    	LightsSchedulerResource lightsSchedulerResource = new LightsSchedulerResource();

		environment.jersey().register(relaysResource);
    	environment.jersey().register(lightsSchedulerResource);
    	environment.jersey().register(new StartPageRedirector());
    	environment.jersey().register(new RelayServiceUIResource(new SysInfoResource(lightsSchedulerResource)));
    	lightsSchedulerResource.start(new LightsSchedulerResource.Listener() {
			@Override
			public void handleLightsEvent(boolean switchLightsOn) {
    			for (Relay relay : relaysResource.getRelays()) {
					relaysResource.powerRelay(relay.getGpio(), !switchLightsOn);
				}
			}    		
    	});    	
    }
    
    private GpioPinDigitalOutput createGpioPinDigitalOutput(GpioController gpioController, Pin pin) {
    	GpioPinDigitalOutput gpioPinDigitalOutput = gpioController.provisionDigitalOutputPin(
    			pin, "Relay" + pin.getAddress(), PinState.LOW);
    	gpioPinDigitalOutput.setShutdownOptions(true, PinState.LOW);
    	return gpioPinDigitalOutput;
    }
    

    protected GpioController createGpioController() {
    	try {
        	return GpioFactory.getInstance();
    	} catch (UnsatisfiedLinkError e) {
    		LOGGER.warn("Could not find native shared object. " +
    				"Assuming not running on a Raspberry Pi. Will run in simulated mode");
    		return new SimulatedGpioController();
    	}
    }
}
