package se.paddbacken.relayservice;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.Collections;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;

import com.codahale.metrics.health.HealthCheck;
import com.codahale.metrics.health.HealthCheckRegistry;
import com.pi4j.io.gpio.GpioController;
import com.pi4j.io.gpio.GpioPinDigitalOutput;
import com.pi4j.io.gpio.Pin;
import com.pi4j.io.gpio.PinState;

import io.dropwizard.Configuration;
import io.dropwizard.jersey.setup.JerseyEnvironment;
import io.dropwizard.jetty.HttpConnectorFactory;
import io.dropwizard.lifecycle.setup.LifecycleEnvironment;
import io.dropwizard.server.DefaultServerFactory;
import io.dropwizard.setup.Environment;
import se.paddbacken.relayservice.scheduling.LightsSchedulerResource;
import se.paddbacken.relayservice.ui.RelayServiceUIResource;

public class RelayServiceApplicationTest {
	private RelayServiceApplication application;
	private GpioController gpioController;
	private GpioPinDigitalOutput pinDigitalOutput;
	private Environment environment;
	private HealthCheckRegistry healthCheckRegistry;
	private JerseyEnvironment jerseyEnvironment;
	private Configuration configuration;
	
	@Before
	public void setup() {
		environment = mock(Environment.class);
		configuration = mock(Configuration.class);
		DefaultServerFactory serverFactory = mock(DefaultServerFactory.class);
		HttpConnectorFactory connectorFactory = mock(HttpConnectorFactory.class);		
		gpioController = mock(GpioController.class);
		pinDigitalOutput = mock(GpioPinDigitalOutput.class);
		healthCheckRegistry = mock(HealthCheckRegistry.class);
		jerseyEnvironment = mock(JerseyEnvironment.class);		
		Pin pin0 = mock(Pin.class);
		Pin pin2 = mock(Pin.class);	
		LifecycleEnvironment lifecycle = mock(LifecycleEnvironment.class);
		application = new RelayServiceApplication() {
			@Override
			protected GpioController createGpioController() {
				return gpioController;
			}
		};
				
		when(configuration.getServerFactory()).thenReturn(serverFactory);
		when(serverFactory.getApplicationConnectors()).thenReturn(
				Collections.singletonList(connectorFactory));		
		when(gpioController.provisionDigitalOutputPin(
				any(Pin.class), anyString(), eq(PinState.LOW))).thenReturn(pinDigitalOutput);
		when(environment.healthChecks()).thenReturn(healthCheckRegistry);
		when(pinDigitalOutput.getPin()).thenReturn(pin0, pin2);
		when(pin0.getAddress()).thenReturn(0);
		when(pin2.getAddress()).thenReturn(2);
		when(environment.jersey()).thenReturn(jerseyEnvironment);
		when(environment.lifecycle()).thenReturn(lifecycle);
	}
	
	@Test
	public void shouldReturnSensibleNameWhenGettingName() {
		assertEquals(RelayServiceApplication.APPLICATION_NAME, application.getName());
	}
	
	@Test
	public void shouldRegisterHealthCheckTwoPinRelaysResourceUiResourceLightsSchedulerAndRemouteArduinoResuorceWhenRunningApp() {
		application.run(configuration, environment);
		verify(healthCheckRegistry).register(eq(RelayServiceApplication.HEALTH_CHECK_NAME),
				any(HealthCheck.class));
		ArgumentCaptor<Object> argumentCaptor = ArgumentCaptor.forClass(Object.class);		
		verify(jerseyEnvironment, times(4)).register(argumentCaptor.capture());
		List<Object> resources = argumentCaptor.getAllValues();
		assertTrue(resources.get(0) instanceof RelaysResource);
		assertTrue(resources.get(1) instanceof LightsSchedulerResource);
		assertTrue(resources.get(2) instanceof StartPageRedirector);
		assertTrue(resources.get(3) instanceof RelayServiceUIResource);	
	}
}
